import os


class SettingsReader():

    def __init__(self, variables):
        """
        variables is a tuple of var_name, setting_name, var_type, default:

         * var_name is the name of the environment variable.
         * setting_name is the name for the setting in the returned dict
         * var_type can be int, str, bool
         * default is optional, and will be returned if the environment
           variable is undefined.
        """

        for variable in variables:
            var_name = variable[0]
            setting_name = variable[1]
            var_type = variable[2]

            if var_name not in os.environ:
                if len(variable) >= 4:
                    setattr(self, setting_name, variable[3])
                    return

                raise Exception("Environment variable {} not set."
                                .format(var_name))

            var = os.environ[var_name]

            if var_type == str:
                setattr(self, setting_name, var)
            elif var_type == int:
                setattr(self, setting_name, int(var))
            elif var_type == bool:
                setattr(self, setting_name, var.lower()
                        in ["true", "yes", "on"])


if __name__ == "__main__":
    """
    Tests this script. Run test.sh to set the proper variables instead of
    running this manually.
    """

    c = SettingsReader([
                       ("A", "a", bool),
                       ("B", "b", bool),
                       ("C", "c", int),
                       ("D", "d", str),
                       ("E", "e", str),
                       ("F", "f", str),
                       ("G", "g", str, "undefined"),
                       ])

    assert(c.a is True)
    assert(c.b is False)
    assert(c.c == 17)
    assert(c.d == "18")
    assert(c.e == "yes")
    assert(c.f == "Some longer text")
    assert(c.g == "undefined")
